from typing import Union, List
from krex.utils.common import Common
from ._http_manager import HTTPManager
from .endpoints.order import Order


class TradeHTTP(HTTPManager):
    async def place_order(
        self,
        product_symbol: str,
        side: str,
        orderQty: int = None,
        ordType: str = "Limit",
        price: float = None,
        stopPx: float = None,
        clOrdID: str = None,
        clOrdLinkID: str = None,
        contingencyType: str = None,
        displayQty: int = None,
        execInst: str = None,
        pegOffsetValue: float = None,
        pegPriceType: str = None,
        timeInForce: str = None,
        text: str = None,
        targetAccountId: int = None,
    ):
        """
        :param product_symbol: str - Trading symbol (required)
        :param side: str - Order side ("Buy", "Sell")
        :param orderQty: int - Order quantity (in contracts)
        :param ordType: str - Order type:
            - "Limit": Limit order (default)
            - "Market": Market order
            - "Stop": Stop market order
            - "StopLimit": Stop limit order
            - "MarketIfTouched": Market if touched order
            - "LimitIfTouched": Limit if touched order
            - "Pegged": Pegged order
            - "Block": Iceberg order
            - "MarketWithLeftOverAsLimit": Market with leftover as limit
        :param price: float - Limit price (for Limit, StopLimit, LimitIfTouched orders)
        :param stopPx: float - Trigger price (for Stop, StopLimit, MarketIfTouched, LimitIfTouched orders)
        :param clOrdID: str - Client order ID (max 36 characters)
        :param clOrdLinkID: str - Client order link ID (for order relationships)
        :param contingencyType: str - Contingency order type:
            - "OneCancelsTheOther": OCO order
            - "OneTriggersTheOther": OTO order
        :param displayQty: int - Display quantity (0 for fully hidden)
        :param execInst: str - Execution instructions:
            - "ParticipateDoNotInitiate": Post-only
            - "AllOrNone": Fill or kill
            - "MarkPrice": Use mark price
            - "IndexPrice": Use index price
            - "LastPrice": Use last price
            - "Close": Close position
            - "ReduceOnly": Reduce only
            - "Fixed": Fixed price (for pegged orders)
            - "LastWithinMark": Last price within mark price
        :param pegOffsetValue: float - Peg offset value
        :param pegPriceType: str - Peg price type:
            - "MarketPeg": Relative to far touch price
            - "PrimaryPeg": Relative to near touch price
            - "TrailingStopPeg": Trailing stop peg
            - "MidPricePeg": Mid price peg
            - "LastPeg": Last price peg
        :param timeInForce: str - Time in force:
            - "Day": Day only
            - "GoodTillCancel": Good till cancelled (default)
            - "ImmediateOrCancel": Immediate or cancel
            - "FillOrKill": Fill or kill
            - "AtTheClose": At the close
        :param text: str - Order annotation
        :param targetAccountId: int - Target account ID
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMEX, product_symbol),
            "side": side,
            "ordType": ordType,
        }

        if orderQty is not None:
            payload["orderQty"] = orderQty
        if price is not None:
            payload["price"] = price
        if stopPx is not None:
            payload["stopPx"] = stopPx
        if clOrdID is not None:
            payload["clOrdID"] = clOrdID
        if clOrdLinkID is not None:
            payload["clOrdLinkID"] = clOrdLinkID
        if contingencyType is not None:
            payload["contingencyType"] = contingencyType
        if displayQty is not None:
            payload["displayQty"] = displayQty
        if execInst is not None:
            payload["execInst"] = execInst
        if pegOffsetValue is not None:
            payload["pegOffsetValue"] = pegOffsetValue
        if pegPriceType is not None:
            payload["pegPriceType"] = pegPriceType
        if timeInForce is not None:
            payload["timeInForce"] = timeInForce
        if text is not None:
            payload["text"] = text
        if targetAccountId is not None:
            payload["targetAccountId"] = targetAccountId

        res = await self._request(
            method="POST",
            path=Order.PLACE_ORDER,
            query=payload,
        )
        return res

    async def place_market_order(
        self,
        product_symbol: str,
        side: str,
        orderQty: int,
        clOrdID: str = None,
    ):
        return await self.place_order(
            product_symbol=product_symbol,
            side=side,
            orderQty=orderQty,
            ordType="Market",
            clOrdID=clOrdID,
        )

    async def place_market_buy_order(
        self,
        product_symbol: str,
        orderQty: int,
        clOrdID: str = None,
    ):
        return await self.place_market_order(
            product_symbol=product_symbol,
            side="Buy",
            orderQty=orderQty,
            clOrdID=clOrdID,
        )

    async def place_market_sell_order(
        self,
        product_symbol: str,
        orderQty: int,
        clOrdID: str = None,
        execInst: str = None,
    ):
        return await self.place_market_order(
            product_symbol=product_symbol,
            side="Sell",
            orderQty=orderQty,
            clOrdID=clOrdID,
        )

    async def place_limit_order(
        self,
        product_symbol: str,
        side: str,
        orderQty: int,
        price: float,
        clOrdID: str = None,
        timeInForce: str = "GoodTillCancel",
    ):
        return await self.place_order(
            product_symbol=product_symbol,
            side=side,
            orderQty=orderQty,
            ordType="Limit",
            price=price,
            clOrdID=clOrdID,
            timeInForce=timeInForce,
        )

    async def place_limit_buy_order(
        self,
        product_symbol: str,
        orderQty: int,
        price: float,
        clOrdID: str = None,
    ):
        return await self.place_limit_order(
            product_symbol=product_symbol,
            side="Buy",
            orderQty=orderQty,
            price=price,
            clOrdID=clOrdID,
        )

    async def place_limit_sell_order(
        self,
        product_symbol: str,
        orderQty: int,
        price: float,
        clOrdID: str = None,
    ):
        return await self.place_limit_order(
            product_symbol=product_symbol,
            side="Sell",
            orderQty=orderQty,
            price=price,
            clOrdID=clOrdID,
        )

    async def place_post_only_order(
        self,
        product_symbol: str,
        side: str,
        orderQty: int,
        price: float,
        clOrdID: str = None,
        execInst: str = "ParticipateDoNotInitiate",
    ):
        return await self.place_order(
            product_symbol=product_symbol,
            side=side,
            orderQty=orderQty,
            ordType="Limit",
            price=price,
            clOrdID=clOrdID,
            execInst=execInst,
        )

    async def place_post_only_buy_order(
        self,
        product_symbol: str,
        orderQty: int,
        price: float,
        clOrdID: str = None,
    ):
        return await self.place_post_only_order(
            product_symbol=product_symbol,
            side="Buy",
            orderQty=orderQty,
            price=price,
            clOrdID=clOrdID,
        )

    async def place_post_only_sell_order(
        self,
        product_symbol: str,
        orderQty: int,
        price: float,
        clOrdID: str = None,
    ):
        return await self.place_post_only_order(
            product_symbol=product_symbol,
            side="Sell",
            orderQty=orderQty,
            price=price,
            clOrdID=clOrdID,
        )

    async def amend_order(
        self,
        orderID: str = None,
        origClOrdID: str = None,
        product_symbol: str = None,
        clOrdID: str = None,
        leavesQty: int = None,
        orderQty: int = None,
        price: float = None,
        stopPx: float = None,
        pegOffsetValue: float = None,
        text: str = None,
        targetAccountId: int = None,
    ):
        """
        :param orderID: str - Order ID to amend (required if origClOrdID not provided)
        :param origClOrdID: str - Client Order ID to amend (required if orderID not provided)
        :param product_symbol: str - Instrument symbol e.g. 'XBTUSD'
        :param clOrdID: str - Optional new Client Order ID, requires origClOrdID
        :param leavesQty: int - Optional leaves quantity in units of the instrument. Useful for amending partially filled orders
        :param orderQty: int - Optional order quantity in units of the instrument
        :param price: float - Optional limit price for 'Limit', 'StopLimit', and 'LimitIfTouched' orders
        :param stopPx: float - Optional trigger price for 'Stop', 'StopLimit', 'MarketIfTouched', and 'LimitIfTouched' orders
        :param pegOffsetValue: float - Optional trailing offset from the current price for 'Stop', 'StopLimit', 'MarketIfTouched', and 'LimitIfTouched' orders
        :param text: str - Optional order annotation. e.g. 'Take profit'
        :param targetAccountId: int - Target account ID
        """
        if orderID is None and origClOrdID is None:
            raise ValueError("Either orderID or origClOrdID must be provided")

        payload = {}

        if orderID is not None:
            payload["orderID"] = orderID
        if origClOrdID is not None:
            payload["origClOrdID"] = origClOrdID
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMEX, product_symbol)
        if clOrdID is not None:
            payload["clOrdID"] = clOrdID
        if leavesQty is not None:
            payload["leavesQty"] = leavesQty
        if orderQty is not None:
            payload["orderQty"] = orderQty
        if price is not None:
            payload["price"] = price
        if stopPx is not None:
            payload["stopPx"] = stopPx
        if pegOffsetValue is not None:
            payload["pegOffsetValue"] = pegOffsetValue
        if text is not None:
            payload["text"] = text
        if targetAccountId is not None:
            payload["targetAccountId"] = targetAccountId

        res = await self._request(
            method="PUT",
            path=Order.AMEND_ORDER,
            query=payload,
        )
        return res

    async def cancel_order(
        self,
        orderID: Union[str, List[str]] = None,
        clOrdID: Union[str, List[str]] = None,
        targetAccountId: int = None,
        text: str = None,
    ):
        """
        :param orderID: Union[str, List[str]] - Order ID(s) to cancel
        :param clOrdID: Union[str, List[str]] - Client Order ID(s) to cancel
        :param targetAccountId: int - Account ID on which to cancel these orders
        :param text: str - Optional order annotation. e.g. 'Take profit'
        """
        payload = {}

        if orderID is not None:
            payload["orderID"] = orderID
        if clOrdID is not None:
            payload["clOrdID"] = clOrdID
        if targetAccountId is not None:
            payload["targetAccountId"] = targetAccountId
        if text is not None:
            payload["text"] = text

        res = await self._request(
            method="DELETE",
            path=Order.CANCEL_ORDER,
            query=payload,
        )
        return res

    async def cancel_all_orders(
        self,
        product_symbol: str = None,
        filter: dict = None,
        targetAccountId: int = None,
        targetAccountIds: list = None,
        text: str = None,
    ):
        """
        :param product_symbol: str
        :param filter: dict (Optional filter for cancellation. Use to only cancel some orders, e.g. {"side": "Buy"}.)
        :param targetAccountId: int
        :param targetAccountIds: list
        :param text: str (Optional cancellation annotation. e.g. 'Spread Exceeded')
        """
        payload = {}

        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMEX, product_symbol)
        if filter is not None:
            payload["filter"] = filter
        if targetAccountId is not None:
            payload["targetAccountId"] = targetAccountId
        if targetAccountIds is not None:
            payload["targetAccountIds"] = targetAccountIds
        if text is not None:
            payload["text"] = text

        res = await self._request(
            method="DELETE",
            path=Order.CANCEL_ALL_ORDERS,
            query=payload,
        )
        return res

    async def get_order(
        self,
        product_symbol: str = None,
        targetAccountId: int = None,
        filter: str = None,
        columns: str = None,
        count: int = 100,
        start: int = 0,
        reverse: bool = False,
        startTime: str = None,
        endTime: str = None,
        targetAccountIds: str = None,
        targetAccountIds_array: list = None,
    ):
        """
        :param product_symbol: str
        :param targetAccountId: int
        :param filter: str
        :param columns: str
        :param count: int
        :param start: int
        :param reverse: bool
        :param startTime: str
        :param endTime: str
        :param targetAccountIds: str
        :param targetAccountIds_array: list
        """
        payload = {}

        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMEX, product_symbol)
        if targetAccountId is not None:
            payload["targetAccountId"] = targetAccountId
        if filter is not None:
            payload["filter"] = filter
        if columns is not None:
            payload["columns"] = columns
        if count is not None:
            payload["count"] = count
        if start is not None:
            payload["start"] = start
        if reverse is not None:
            payload["reverse"] = reverse
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime
        if targetAccountIds is not None:
            payload["targetAccountIds"] = targetAccountIds
        if targetAccountIds_array is not None:
            payload["targetAccountIds[]"] = targetAccountIds_array

        res = await self._request(
            method="GET",
            path=Order.QUERY_ORDER,
            query=payload,
        )
        return res



from ._http_manager import HTTPManager
from .endpoints.market import SwapMarket
from ...utils.common import Common


class MarketHTTP(HTTPManager):
    async def get_swap_instrument_info(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINGX, product_symbol)

        res = await self._request(
            method="GET",
            path=SwapMarket.INSTRUMENT_INFO,
            query=payload,
            signed=False,
        )
        return res

    async def get_orderbook(
        self,
        product_symbol: str,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param limit: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
        }
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SwapMarket.ORDERBOOK,
            query=payload,
            signed=False,
        )
        return res

    async def get_public_trades(
        self,
        product_symbol: str,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param limit: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
        }
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SwapMarket.PUBLIC_TRADE,
            query=payload,
            signed=False,
        )
        return res

    async def get_kline(
        self,
        product_symbol: str,
        interval: str,
        start_time: int = None,
        end_time: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param interval: str
        :param start_time: int
        :param end_time: int
        :param limit: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINGX, product_symbol),
            "interval": interval,
        }
        if start_time is not None:
            payload["startTime"] = start_time
        if end_time is not None:
            payload["endTime"] = end_time
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SwapMarket.KLINE,
            query=payload,
            signed=False,
        )
        return res

    async def get_ticker(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINGX, product_symbol)

        res = await self._request(
            method="GET",
            path=SwapMarket.TICKER,
            query=payload,
            signed=False,
        )
        return res



from enum import Enum


class SpotTrade(str, Enum):
    PLACE_ORDER = "/api/v1/hf/orders"
    BATCH_ORDERS = "/api/v1/hf/orders/multi"
    CANCEL_ORDER = "/api/v1/hf/orders/{orderId}"
    CANCEL_ALL_ORDERS_BY_SYMBOL = "/api/v1/hf/orders"
    CANCEL_ALL_ORDERS = "/api/v1/hf/orders/cancelAll"

    GET_OPEN_ORDERS = "/api/v1/hf/orders/active"
    GET_TRADE_HISTORY = "/api/v1/hf/fills"

    def __str__(self) -> str:
        return self.value



from dataclasses import dataclass
from ._trade_http import TradeHTTP
from ._account_http import AccountHTTP
from ._market_http import MarketHTTP


@dataclass
class Client(
    TradeHTTP,
    AccountHTTP,
    MarketHTTP,
):
    def __init__(self, **args):
        super().__init__(**args)



from krex.utils.common import Common
from ._http_manager import HTTPManager
from .endpoints.market import SpotMarket
from ...utils.timeframe_utils import bybit_convert_timeframe


class MarketHTTP(HTTPManager):
    async def get_spot_instrument_info(
        self,
    ):
        payload = {}
        res = await self._request(
            method="GET",
            path=SpotMarket.INSTRUMENT_INFO,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_ticker(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol)

        res = await self._request(
            method="GET",
            path=SpotMarket.TICKER,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_kline(
        self,
        product_symbol: str,
        interval: str,
        to_timestamp: int = None,
        from_timestamp: int = None,
        n: int = None,
    ):
        """
        :param product_symbol: str
        :param interval: str
        :param to_timestamp: int
        :param from_timestamp: int
        :param n: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol),
            "interval": bybit_convert_timeframe(interval),
        }
        if to_timestamp is not None:
            payload["to"] = to_timestamp
        if from_timestamp is not None:
            payload["from"] = from_timestamp
        if n is not None:
            payload["n"] = n

        res = await self._request(
            method="GET",
            path=SpotMarket.KLINE,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_orderbook(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol),
        }

        res = await self._request(
            method="GET",
            path=SpotMarket.ORDERBOOK,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_public_trade(
        self,
        product_symbol: str,
        n: int = None,
    ):
        """
        :param product_symbol: str
        :param n: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.ASCENDEX, product_symbol),
        }
        if n is not None:
            payload["n"] = n

        res = await self._request(
            method="GET",
            path=SpotMarket.PUBLIC_TRADE,
            query=payload,
            signed=False,
        )
        return res



from enum import Enum


class Market(str, Enum):
    INSTRUMENT_INFO = "/api/v1/instrument/active"
    ORDERBOOK = "/api/v1/orderBook/L2"
    TRADE = "/api/v1/trade"
    TICKER = "/api/v1/quote/bucketed"
    KLINE = "/api/v1/trade/bucketed"
    FUNDING = "/api/v1/funding"

    def __str__(self) -> str:
        return self.value



import hmac
import hashlib
import logging
import json
import httpx
from dataclasses import dataclass, field

from krex.utils.common import Common
from ..product_table.manager import ProductTableManager
from ...utils.errors import FailedRequestError
from ...utils.helpers import generate_timestamp
from .endpoints.account import CashAccount

HTTP_URL = "https://ascendex.com"


def get_header(api_key, signature, timestamp):
    return {
        "Content-Type": "application/json",
        "x-auth-key": str(api_key),
        "x-auth-timestamp": str(timestamp),
        "x-auth-signature": str(signature),
    }


def get_header_no_sign():
    return {"Content-Type": "application/json"}


@dataclass
class HTTPManager:
    api_key: str = field(default=None)
    api_secret: str = field(default=None)
    timeout: int = field(default=10)
    max_retries: int = field(default=3)
    retry_delay: int = field(default=3)
    logger: logging.Logger = field(default=None)
    session: httpx.AsyncClient = field(init=False)
    ptm: ProductTableManager = field(init=False)
    preload_product_table: bool = field(default=True)
    account_group: str = field(default=None)

    async def async_init(self):
        self.session = httpx.AsyncClient(timeout=self.timeout)
        self._logger = self.logger or logging.getLogger(__name__)
        if self.preload_product_table:
            self.ptm = await ProductTableManager.get_instance(Common.ASCENDEX)
        self.endpoint = HTTP_URL

        return self

    def _auth(self, path, timestamp):
        """Generate signature for AscendEX API
        Signature format: HMAC-SHA256(timestamp + path, secret_key)
        """
        param_str = f"{timestamp}+{path}"
        return hmac.new(self.api_secret.encode(), param_str.encode(), hashlib.sha256).hexdigest()

    async def _fetch_account_group_raw(self):
        url = self.endpoint + CashAccount.ACCOUNT_INFO
        timestamp = generate_timestamp()
        sign_path = CashAccount.ACCOUNT_INFO.hash
        signature = self._auth(sign_path, timestamp)
        headers = get_header(self.api_key, signature, timestamp)
        async with httpx.AsyncClient(timeout=self.timeout) as session:
            response = await session.get(url, headers=headers)
            data = response.json()
            if data.get("code", 0) == 0 and "data" in data:
                return data["data"].get("accountGroup")
            else:
                raise ValueError("Failed to fetch account group: " + str(data))

    async def _request(
        self,
        method: str,
        path: str,
        query: dict = None,
        signed: bool = True,
        hash_path: str = None,
    ):
        if not self.session:
            await self.async_init()

        path = await self._resolve_path(path)

        timestamp = generate_timestamp()

        url = self.endpoint + path
        payload = ""

        if method.upper() == "GET" or method.upper() == "DELETE":
            if query:
                sorted_query = "&".join(f"{k}={v}" for k, v in sorted(query.items()) if v)
                url += "?" + sorted_query if sorted_query else ""
                payload = sorted_query
        else:
            payload = json.dumps(query, separators=(",", ":"), ensure_ascii=False)

        if signed:
            if not (self.api_key and self.api_secret):
                raise ValueError("Signed request requires API Key and Secret.")

            sign_path = hash_path if hash_path else path
            signature = self._auth(sign_path, timestamp)
            headers = get_header(self.api_key, signature, timestamp)
        else:
            headers = get_header_no_sign()

        try:
            if method.upper() == "GET":
                response = await self.session.get(url, headers=headers)
            elif method.upper() == "POST":
                response = await self.session.post(url, headers=headers, json=query if query else {})
            elif method.upper() == "DELETE":
                response = await self.session.delete(url, headers=headers)
            else:
                raise ValueError(f"Unsupported HTTP method: {method}")

            try:
                data = response.json()
            except Exception:
                data = {}

            if data.get("code", 0) != 0:
                code = data.get("code", "Unknown")
                error_message = data.get("message", "Unknown error")
                raise FailedRequestError(
                    request=f"{method.upper()} {url} | Body: {query}",
                    message=f"AscendEX API Error: [{code}] {error_message}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            if not response.status_code // 100 == 2:
                raise FailedRequestError(
                    request=f"{method.upper()} {url} | Body: {query}",
                    message=f"HTTP Error {response.status_code}: {response.text}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            return data

        except httpx.ConnectError as e:
            # Handle connection errors specifically
            raise FailedRequestError(
                request=f"{method.upper()} {url} | Body: {payload}",
                message=f"Connection failed: {str(e)}",
                status_code="Connection Error",
                time=timestamp,
                resp_headers=None,
            )
        except httpx.HTTPError as e:
            # Handle other HTTP errors
            status_code = getattr(e.request, "status_code", "Unknown") if hasattr(e, "response") else "Unknown"
            resp_headers = getattr(e.request, "headers", None) if hasattr(e, "response") else None

            raise FailedRequestError(
                request=f"{method.upper()} {url} | Body: {payload}",
                message=f"Request failed: {str(e)}",
                status_code=status_code,
                time=timestamp,
                resp_headers=resp_headers,
            )

    async def _resolve_path(self, path: str) -> str:
        """Resolve dynamic path parameters like {ACCOUNT_GROUP}"""
        if "{ACCOUNT_GROUP}" in path:
            if not self.account_group:
                self.account_group = await self._fetch_account_group_raw()
            return path.replace("{ACCOUNT_GROUP}", str(self.account_group))
        return path



from enum import Enum


class Positions(str, Enum):
    GET_POSITIONS = "/api/v1/position"
    SWITCH_MODE = "/api/v1/position/isolate"
    LEVERAGE = "/api/v1/position/leverage"
    MARGINING_MODE = "/api/v1/user/marginingMode"
    GET_MARGIN = "/api/v1/user/margin"

    def __str__(self) -> str:
        return self.value



import hmac
import hashlib
import json
import time
import httpx
import logging
from dataclasses import dataclass, field
from urllib.parse import urlencode
import requests
from krex.utils.common import Common
from krex.utils.errors import FailedRequestError
from krex.utils.helpers import generate_timestamp
from krex.product_table.manager import ProductTableManager


@dataclass
class HTTPManager:
    api_key: str = field(default=None)
    api_secret: str = field(default=None)
    timeout: int = field(default=30)
    logger: logging.Logger = field(default=None)
    session: requests.Session = field(default_factory=requests.Session, init=False)
    ptm: ProductTableManager = field(default=None, init=False)
    preload_product_table: bool = field(default=True)

    # Bitmex API base URL
    base_url: str = "https://www.bitmex.com"

    def __post_init__(self):
        if self.logger is None:
            self._logger = logging.getLogger(__name__)
        else:
            self._logger = self.logger

        if self.preload_product_table:
            self.ptm = ProductTableManager.get_instance(Common.BITMEX)

    def _sign(self, method: str, path: str, expires: int, body: str = "") -> str:
        """Generate Bitmex API signature according to BitMEX documentation"""
        message = method + path + str(expires) + body
        signature = hmac.new(self.api_secret.encode("utf-8"), message.encode("utf-8"), hashlib.sha256).hexdigest()
        return signature

    def _headers(self, method: str, path: str, body: str = "", signed: bool = True):
        """Generate headers for Bitmex API"""
        headers = {"Content-Type": "application/json", "Accept": "application/json"}

        if self.api_key and self.api_secret and signed:
            expires = int(time.time()) + 5  # 5 seconds from now
            signature = self._sign(method, path, expires, body)
            headers.update({"api-key": self.api_key, "api-signature": signature, "api-expires": str(expires)})

        return headers

    def _request(
        self,
        method,
        path: str,
        query: dict = None,
        signed: bool = True,
    ):
        response = None
        try:
            url = f"{self.base_url}{path}"
            body = ""
            full_path = path

            if method.upper() == "GET":
                if query:
                    query_string = urlencode(query)
                    url += f"?{query_string}"
                    full_path += f"?{query_string}"
                response = self.session.get(url, headers=self._headers(method, full_path, signed=signed))
            elif method.upper() == "POST":
                body = json.dumps(query, separators=(",", ":")) if query else ""
                response = self.session.post(
                    url,
                    headers=self._headers(method, full_path, body, signed=signed),
                    data=body,
                )
            elif method.upper() == "PUT":
                body = json.dumps(query, separators=(",", ":")) if query else ""
                response = self.session.put(
                    url,
                    headers=self._headers(method, full_path, body, signed=signed),
                    data=body,
                )
            elif method.upper() == "DELETE":
                body = json.dumps(query, separators=(",", ":")) if query else ""
                response = self.session.request(
                    method="DELETE",
                    url=url,
                    headers=self._headers(method, full_path, body, signed=signed),
                    data=body,
                )
            else:
                raise ValueError(f"Unsupported method: {method}")

            try:
                data = response.json()
            except Exception:
                data = {}

            timestamp = generate_timestamp(iso_format=True)

            if not response.status_code // 100 == 2:
                error_message = (
                    data.get("error", {}).get("message", "Unknown error") if isinstance(data, dict) else response.text
                )
                raise FailedRequestError(
                    request=f"{method} {url} | Body: {query}",
                    message=f"BITMEX API Error: {error_message}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            return data

        except httpx.RequestError as e:
            raise FailedRequestError(
                request=f"{method.upper()} {url} | Params: {query}",
                message=f"Request failed: {str(e)}",
                status_code=response.status_code if response else "Unknown",
                time=timestamp if "timestamp" in locals() else "Unknown",
                resp_headers=response.headers if response else None,
            )



from enum import Enum


class FundingAccount(str, Enum):
    # https://api-cloud.bitmart.com
    GET_ACCOUNT_BALANCE = "/account/v1/wallet"
    GET_ACCOUNT_CURRENCIES = "/account/v1/currencies"
    GET_SPOT_WALLET_BALANCE = "/spot/v1/wallet"
    DEPOSIT_ADDRESS = "/account/v1/deposit/address"
    WITHDRAW_QUOTA = "/account/v1/withdraw/charge"
    WITHDRAW = "/account/v1/withdraw/apply"
    GET_DEPOSIT_WITHDRAW_HISTORY = "/account/v2/deposit-withdraw/history"
    GET_DEPOSIT_WITHDRAW_HISTORY_DETAIL = "/account/v1/deposit-withdraw/detail"

    def __str__(self) -> str:
        return self.value


class FuturesAccount(str, Enum):
    # https://api-cloud-v2.bitmart.com
    GET_CONTRACT_ASSETS = "/contract/private/assets-detail"

    def __str__(self) -> str:
        return self.value



import hmac
import json
import time
import requests
import logging
import hashlib
from enum import Enum
from dataclasses import dataclass, field
from typing import Optional
from ..product_table.manager import ProductTableManager
from ..utils.errors import FailedRequestError
from ..utils.common import Common


@dataclass
class HTTPManager:
    api_key: str = field(default=None)
    api_secret: str = field(default=None)
    base_url: str = field(default="https://api.gateio.ws")
    logger: logging.Logger = field(default=None)
    session: requests.Session = field(default_factory=requests.Session, init=False)
    preload_product_table: bool = field(default=True)

    def __post_init__(self):
        if self.logger is None:
            self._logger = logging.getLogger(__name__)
        else:
            self._logger = self.logger

        if self.preload_product_table:
            self.ptm = ProductTableManager.get_instance(Common.GATEIO)

    def _resolve_path(self, path_template, path_params: dict = None) -> str:
        if isinstance(path_template, Enum):
            path_template = path_template.value

        try:
            return path_template.format(**(path_params or {}))
        except KeyError as e:
            raise ValueError(f"Missing path parameter: {e}")

    def _sign(self, method: str, url_path: str, query: Optional[dict], body: Optional[dict], timestamp: str) -> str:
        payload_string = json.dumps(body or {}, separators=(",", ":")) if body else ""
        hashed_payload = hashlib.sha512(payload_string.encode("utf-8")).hexdigest()

        query_string = ""
        if query:
            query_string = "&".join(f"{k}={v}" for k, v in sorted(query.items()))

        s = f"{method.upper()}\n{url_path}\n{query_string}\n{hashed_payload}\n{timestamp}"
        return hmac.new(self.api_secret.encode("utf-8"), s.encode("utf-8"), hashlib.sha512).hexdigest()

    def _request(
        self,
        method: str,
        path: str,
        path_params: Optional[dict] = None,
        query: Optional[dict] = None,
        body: Optional[dict] = None,
        signed: bool = True,
    ):
        query = query or {}
        body = body or {}

        resolved_path = self._resolve_path(path, path_params)
        full_path = "/api/v4" + resolved_path
        url = self.base_url + full_path

        timestamp = str(int(time.time()))
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }

        if signed:
            if not (self.api_key and self.api_secret):
                raise ValueError("Signed request requires API Key and Secret.")
            sign = self._sign(method, full_path, query, body, timestamp)
            headers.update(
                {
                    "KEY": self.api_key,
                    "Timestamp": timestamp,
                    "SIGN": sign,
                }
            )

        try:
            method_upper = method.upper()
            body_string = None
            if method_upper in ("POST", "PUT", "PATCH"):
                body_string = json.dumps(body, separators=(",", ":"))

            if method_upper == "GET":
                response = self.session.get(url, headers=headers, params=query)
            elif method_upper == "POST":
                if body:
                    response = self.session.post(url, headers=headers, params=query, data=body_string)
                else:
                    response = self.session.post(url, headers=headers, params=query)
            elif method_upper == "PUT":
                response = self.session.put(url, headers=headers, params=query, data=body_string)
            elif method_upper == "DELETE":
                response = self.session.delete(url, headers=headers, params=query)
            elif method_upper == "PATCH":
                response = self.session.patch(url, headers=headers, params=query, data=body_string)
            else:
                raise ValueError(f"Unsupported HTTP method: {method}")

            if response.ok:
                return response.json()

            raise FailedRequestError(
                request=f"{method_upper} {url}",
                message=f"GATEIO API Error: {response.status_code}, {response.text}",
                status_code=response.status_code,
                time=timestamp,
                resp_headers=dict(response.headers),
            )

        except FailedRequestError:
            raise
        except Exception as e:
            raise FailedRequestError(
                request=f"{method.upper()} {url}",
                message=f"Request failed: {e}",
                status_code="unknown",
                time=timestamp,
                resp_headers={},
            )



from ._http_manager import HTTPManager
from .endpoints.trade import SpotTrade, FuturesTrade
from ..utils.common import Common


class TradeHTTP(HTTPManager):
    def place_spot_order(
        self,
        product_symbol: str,
        side: str,
        type: str,
        size: str = None,
        price: str = None,
        notional: str = None,
        client_order_id: str = None,
    ):
        """
        :param product_symbol: str
        :param side: str (buy, sell)
        :param type: str (limit, market, limit_maker, ioc)
        :param size: str
        :param price: str
        :param notional: str
        :param clientOrderId: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
            "side": side,
            "type": type,
        }
        if size is not None:
            payload["size"] = size
        if price is not None:
            payload["price"] = price
        if notional is not None:
            payload["notional"] = notional
        if client_order_id is not None:
            payload["client_order_id"] = client_order_id

        return self._request(
            method="POST",
            path=SpotTrade.SUBMIT_ORDER,
            query=payload,
        )

    def place_spot_market_order(
        self,
        product_symbol: str,
        side: str,
        size: str = None,
        notional: str = None,
        client_order_id: str = None,
    ):
        return self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            type="market",
            size=size,
            notional=notional,
            client_order_id=client_order_id,
        )

    def place_spot_market_buy_order(
        self,
        product_symbol: str,
        notional: str,
        client_order_id: str = None,
    ):
        return self.place_spot_market_order(
            product_symbol=product_symbol,
            side="buy",
            notional=notional,
            client_order_id=client_order_id,
        )

    def place_spot_market_sell_order(
        self,
        product_symbol: str,
        size: str,
        client_order_id: str = None,
    ):
        return self.place_spot_market_order(
            product_symbol=product_symbol,
            side="sell",
            size=size,
            client_order_id=client_order_id,
        )

    def place_spot_limit_order(
        self,
        product_symbol: str,
        side: str,
        size: str,
        price: str,
        client_order_id: str = None,
    ):
        return self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            type="limit",
            size=size,
            price=price,
            client_order_id=client_order_id,
        )

    def place_spot_limit_buy_order(
        self,
        product_symbol: str,
        size: str,
        price: str,
        client_order_id: str = None,
    ):
        return self.place_spot_limit_order(
            product_symbol=product_symbol,
            side="buy",
            size=size,
            price=price,
            client_order_id=client_order_id,
        )

    def place_spot_limit_sell_order(
        self,
        product_symbol: str,
        size: str,
        price: str,
        client_order_id: str = None,
    ):
        return self.place_spot_limit_order(
            product_symbol=product_symbol,
            side="sell",
            size=size,
            price=price,
            client_order_id=client_order_id,
        )

    def place_spot_post_only_limit_order(
        self,
        product_symbol: str,
        side: str,
        size: str,
        price: str,
        client_order_id: str = None,
    ):
        return self.place_spot_order(
            product_symbol=product_symbol,
            side=side,
            type="limit_maker",
            size=size,
            price=price,
            client_order_id=client_order_id,
        )

    def place_spot_post_only_limit_buy_order(
        self,
        product_symbol: str,
        size: str,
        price: str,
        client_order_id: str = None,
    ):
        return self.place_post_only_limit_order(
            product_symbol=product_symbol,
            side="buy",
            size=size,
            price=price,
            client_order_id=client_order_id,
        )

    def place_post_only_limit_sell_order(
        self,
        product_symbol: str,
        size: str,
        price: str,
        client_order_id: str = None,
    ):
        return self.place_post_only_limit_order(
            product_symbol=product_symbol,
            side="sell",
            size=size,
            price=price,
            clientOrderId=client_order_id,
        )

    def cancel_spot_order(
        self,
        product_symbol: str,
        order_id: str = None,
        client_order_id: str = None,
    ):
        """
        :param product_symbol: str
        :param order_id: str (order_id, client_order_id, one of them is required)
        :param client_order_id: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }
        if order_id is not None:
            payload["order_id"] = order_id
        if client_order_id is not None:
            payload["client_order_id"] = client_order_id

        return self._request(
            method="POST",
            path=SpotTrade.CANCEL_ORDER,
            query=payload,
        )

    def cancel_spot_all_order(
        self,
        product_symbol: str = None,
        side: str = None,
    ):
        """
        :param product_symbol: str
        :param side: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMART, product_symbol)
        if side is not None:
            payload["side"] = side

        return self._request(
            method="POST",
            path=SpotTrade.CANCEL_ALL_ORDERS,
            query=payload,
        )

    def place_margin_order(
        self,
        product_symbol: str,
        side: str,
        type: str,
        size: str = None,
        price: str = None,
        notional: str = None,
        clientOrderId: str = None,
    ):
        """
        :param product_symbol: str
        :param side: str (buy, sell)
        :param type: str (limit, market, limit_maker, ioc)
        :param size: str
        :param price: str
        :param clientOrderId: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
            "side": side,
            "type": type,
        }
        if clientOrderId is not None:
            payload["clientOrderId"] = clientOrderId
        if size is not None:
            payload["size"] = size
        if price is not None:
            payload["price"] = price
        if notional is not None:
            payload["notional"] = notional

        return self._request(
            method="POST",
            path=SpotTrade.NEW_MARGIN_ORDER,
            query=payload,
        )

    def get_spot_order_by_order_id(
        self,
        orderId: str,
        queryState: str = None,
    ):
        """
        :param orderId: str
        :param queryState: str (open, history)
        """
        payload = {
            "orderId": orderId,
        }
        if queryState is not None:
            payload["queryState"] = queryState

        res = self._request(
            method="POST",
            path=SpotTrade.QUERY_ORDER_BY_ID,
            query=payload,
        )

        return res

    def get_spot_order_by_order_client_id(
        self,
        clientOrderId: str,
        queryState: str = None,
    ):
        """
        :param clientOrderId: str
        :param queryState: str (open, history)
        """
        payload = {
            "clientOrderId": clientOrderId,
        }
        if queryState is not None:
            payload["queryState"] = queryState

        res = self._request(
            method="POST",
            path=SpotTrade.QUERY_ORDER_BY_CLIENT_ORDER_ID,
            query=payload,
        )

        return res

    def get_spot_open_orders(
        self,
        product_symbol: str = None,
        orderMode: str = None,
        startTime: int = None,
        endTime: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param orderMode: str (spot, iso_margin)
        :param startTime: int
        :param endTime: int
        :param limit: int
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMART, product_symbol)
        if orderMode is not None:
            payload["orderMode"] = orderMode
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="POST",
            path=SpotTrade.CURRENT_OPEN_ORDERS,
            query=payload,
        )

        return res

    def get_spot_account_orders(
        self,
        product_symbol: str = None,
        orderMode: str = None,
        startTime: int = None,
        endTime: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param orderMode: str (spot, iso_margin)
        :param startTime: int
        :param endTime: int
        :param limit: int
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMART, product_symbol)
        if orderMode is not None:
            payload["orderMode"] = orderMode
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="POST",
            path=SpotTrade.ACCOUNT_ORDERS,
            query=payload,
        )

        return res

    def get_spot_account_trade_list(
        self,
        product_symbol: str = None,
        orderMode: str = None,
        startTime: int = None,
        endTime: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param orderMode: str (spot, iso_margin)
        :param startTime: int
        :param endTime: int
        :param limit: int
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMART, product_symbol)
        if orderMode is not None:
            payload["orderMode"] = orderMode
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="POST",
            path=SpotTrade.ACCOUNT_TRADE_LIST,
            query=payload,
        )

        return res

    def get_spot_order_trade_list(
        self,
        orderId: str,
    ):
        """
        :param orderId: str
        """
        payload = {
            "orderId": orderId,
        }

        res = self._request(
            method="POST",
            path=SpotTrade.ORDER_TRADE_LIST,
            query=payload,
        )

        return res

    def place_contract_order(
        self,
        product_symbol: str,
        side: int,
        size: int,
        price: str = None,
        client_order_id: str = None,
        type: str = None,
        leverage: str = None,
        open_type: str = None,
        mode: int = None,
        preset_take_profit_price_type: int = None,
        preset_stop_loss_price_type: int = None,
        preset_take_profit_price: str = None,
        preset_stop_loss_price: str = None,
        stp_mode: int = None,
    ):
        """
        :param product_symbol: str
        :param side: str (1=buy_open_long, 2=buy_close_short, 3=sell_close_long, 4=sell_open_short)
        :param price: str
        :param size: str
        :param client_order_id: str
        :param type: str (limit(default), market)
        :param leverage: str
        :param open_type: str (cross, isolated)
        :param mode: int (1=GTC(default), 2=FOK, 3=IOC, 4=Maker Only)
        :param preset_take_profit_price_type: int Pre-set TP price type(1=last_price(default), 2=fair_price)
        :param preset_stop_loss_price_type: int Pre-set SL price type(1=last_price(default), 2=fair_price)
        :param preset_take_profit_price: str Pre-set TP price
        :param preset_stop_loss_price: str Pre-set SL price
        :param stp_mode: int (1: cancel_maker(default), 2: cancel_taker, 3: cancel_both)
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
            "side": side,
            "size": size,
        }
        if price is not None:
            payload["price"] = price
        if client_order_id is not None:
            payload["client_order_id"] = client_order_id
        if type is not None:
            payload["type"] = type
        if leverage is not None:
            payload["leverage"] = leverage
        if open_type is not None:
            payload["open_type"] = open_type
        if mode is not None:
            payload["mode"] = mode
        if preset_take_profit_price_type is not None:
            payload["preset_take_profit_price_type"] = preset_take_profit_price_type
        if preset_stop_loss_price_type is not None:
            payload["preset_stop_loss_price_type"] = preset_stop_loss_price_type
        if preset_take_profit_price is not None:
            payload["preset_take_profit_price"] = preset_take_profit_price
        if preset_stop_loss_price is not None:
            payload["preset_stop_loss_price"] = preset_stop_loss_price
        if stp_mode is not None:
            payload["stp_mode"] = stp_mode

        return self._request(
            method="POST",
            path=FuturesTrade.SUBMIT_ORDER,
            query=payload,
        )

    def place_contract_market_order(
        self,
        product_symbol: str,
        side: int,
        size: int,
        client_order_id: str = None,
    ):
        return self.place_contract_order(
            product_symbol=product_symbol,
            side=side,
            type="market",
            size=size,
            client_order_id=client_order_id,
        )

    def place_contract_market_buy_order(
        self,
        product_symbol: str,
        size: int,
        client_order_id: str = None,
    ):
        positions = self.get_contract_position(product_symbol)
        short_size = sum(int(p["current_amount"]) for p in positions if p["position_type"] == 2)

        if short_size != 0:
            excess_size = size - short_size
            if excess_size <= 0:
                return self.place_contract_market_order(
                    product_symbol=product_symbol,
                    side=2,
                    size=size,
                    client_order_id=client_order_id,
                )
            else:
                return (
                    self.place_contract_market_order(
                        product_symbol=product_symbol,
                        side=2,
                        size=short_size,
                        client_order_id=client_order_id,
                    ),
                    self.place_contract_market_order(
                        product_symbol=product_symbol,
                        side=1,
                        size=excess_size,
                        client_order_id=client_order_id,
                    ),
                )
        else:
            return self.place_contract_market_order(
                product_symbol=product_symbol,
                side=1,
                size=size,
                client_order_id=client_order_id,
            )

    def place_contract_market_sell_order(
        self,
        product_symbol: str,
        size: int,
        client_order_id: str = None,
    ):
        positions = self.get_contract_position(product_symbol)
        long_size = sum(int(p["current_amount"]) for p in positions if p["position_type"] == 1)

        if long_size != 0:
            excess_size = size - long_size
            if excess_size <= 0:
                return self.place_contract_market_order(
                    product_symbol=product_symbol,
                    side=3,
                    size=size,
                    client_order_id=client_order_id,
                )
            else:
                return (
                    self.place_contract_market_order(
                        product_symbol=product_symbol,
                        side=3,
                        size=long_size,
                        client_order_id=client_order_id,
                    ),
                    self.place_contract_market_order(
                        product_symbol=product_symbol,
                        side=4,
                        size=excess_size,
                        client_order_id=client_order_id,
                    ),
                )
        else:
            return self.place_contract_market_order(
                product_symbol=product_symbol,
                side=4,
                size=size,
                client_order_id=client_order_id,
            )

    def place_contract_limit_order(
        self,
        product_symbol: str,
        side: int,
        price: str,
        size: int,
        client_order_id: str = None,
        mode: int = None,
    ):
        return self.place_contract_order(
            product_symbol=product_symbol,
            side=side,
            type="limit",
            price=price,
            size=size,
            client_order_id=client_order_id,
            mode=mode,
        )

    def place_contract_post_only_order(
        self,
        product_symbol: str,
        side: int,
        price: str,
        size: int,
        client_order_id: str = None,
    ):
        return self.place_contract_limit_order(
            product_symbol=product_symbol,
            side=side,
            price=price,
            size=size,
            client_order_id=client_order_id,
            mode=4,
        )

    def place_contract_post_only_buy_order(
        self,
        product_symbol: str,
        price: str,
        size: int,
        client_order_id: str = None,
    ):
        positions = self.get_contract_position(product_symbol)
        short_size = sum(int(p["current_amount"]) for p in positions if p["position_type"] == 2)

        if short_size != 0:
            excess_size = size - short_size
            if excess_size <= 0:
                return self.place_contract_post_only_order(
                    product_symbol=product_symbol,
                    side=2,
                    price=price,
                    size=size,
                    client_order_id=client_order_id,
                )
            else:
                return (
                    self.place_contract_post_only_order(
                        product_symbol=product_symbol,
                        side=2,
                        price=price,
                        size=short_size,
                        client_order_id=client_order_id,
                    ),
                    self.place_contract_post_only_order(
                        product_symbol=product_symbol,
                        side=1,
                        price=price,
                        size=excess_size,
                        client_order_id=client_order_id,
                    ),
                )
        else:
            return self.place_contract_post_only_order(
                product_symbol=product_symbol,
                side=1,
                price=price,
                size=size,
                client_order_id=client_order_id,
            )

    def place_contract_post_only_sell_order(
        self,
        product_symbol: str,
        price: str,
        size: int,
        client_order_id: str = None,
    ):
        positions = self.get_contract_position(product_symbol)
        long_size = sum(int(p["current_amount"]) for p in positions if p["position_type"] == 1)

        if long_size != 0:
            excess_size = size - long_size
            if excess_size <= 0:
                return self.place_contract_post_only_order(
                    product_symbol=product_symbol,
                    side=3,
                    price=price,
                    size=size,
                    client_order_id=client_order_id,
                )
            else:
                return (
                    self.place_contract_post_only_order(
                        product_symbol=product_symbol,
                        side=3,
                        price=price,
                        size=long_size,
                        client_order_id=client_order_id,
                    ),
                    self.place_contract_post_only_order(
                        product_symbol=product_symbol,
                        side=4,
                        price=price,
                        size=excess_size,
                        client_order_id=client_order_id,
                    ),
                )
        else:
            return self.place_contract_post_only_order(
                product_symbol=product_symbol,
                side=4,
                price=price,
                size=size,
                client_order_id=client_order_id,
            )

    def modify_limit_order(
        self,
        product_symbol: str,
        order_id: str = None,
        client_order_id: str = None,
        price: int = None,
        size: int = None,
    ):
        """
        :param product_symbol: str
        :param order_id: str
        :param client_order_id: str
        :param price: int
        :param size: int
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }
        if order_id is not None:
            payload["order_id"] = order_id
        if client_order_id is not None:
            payload["client_order_id"] = client_order_id
        if price is not None:
            payload["price"] = price
        if size is not None:
            payload["size"] = size

        return self._request(
            method="POST",
            path=FuturesTrade.MODIFY_LIMIT_ORDER,
            query=payload,
        )

    def cancel_contract_order(
        self,
        product_symbol: str,
        order_id: str = None,
        client_order_id: str = None,
    ):
        """
        :param product_symbol: str
        :param order_id: str
        :param client_order_id: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(product_symbol, Common.BITMART),
        }
        if order_id is not None:
            payload["order_id"] = order_id
        if client_order_id is not None:
            payload["client_order_id"] = client_order_id

        return self._request(
            method="POST",
            path=FuturesTrade.CANCEL_ORDER,
            query=payload,
        )

    def cancel_all_contract_order(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }

        return self._request(
            method="POST",
            path=FuturesTrade.CANCEL_ALL_ORDERS,
            query=payload,
        )

    def transfer_contract(
        self,
        amount: str,
        type: str,
    ):
        """
        :param currency: str
        :param amount: str
        :param type: str (spot_to_contract, contract_to_spot)
        """
        payload = {
            "currency": "USDT",
            "amount": amount,
            "type": type,
        }

        return self._request(
            method="POST",
            path=FuturesTrade.TRANSFER,
            query=payload,
        )

    def submit_leverage(
        self,
        product_symbol: str,
        leverage: str = None,
        open_type: str = None,
    ):
        """
        :param product_symbol: str
        :param leverage: str
        :param open_type: str (cross, isolated)
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }
        if leverage is not None:
            payload["leverage"] = leverage
        if open_type is not None:
            payload["open_type"] = open_type

        return self._request(
            method="POST",
            path=FuturesTrade.SUBMIT_LEVERAGE,
            query=payload,
        )

    def get_contract_order_detail(
        self,
        product_symbol: str,
        order_id: str,
    ):
        """
        :param product_symbol: str
        :param order_id: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(product_symbol, Common.BITMART),
            "order_id": order_id,
        }

        res = self._request(
            method="GET",
            path=FuturesTrade.GET_ORDER_DETAIL,
            query=payload,
        )

        return res

    def get_contract_order_history(
        self,
        product_symbol: str,
        start_time: str = None,
        end_time: str = None,
    ):
        """
        :param product_symbol: str
        :param start_time: str
        :param end_time: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(product_symbol, Common.BITMART),
        }
        if start_time is not None:
            payload["start_time"] = start_time
        if end_time is not None:
            payload["end_time"] = end_time

        res = self._request(
            method="GET",
            path=FuturesTrade.GET_ORDER_HISTORY,
            query=payload,
        )

        return res

    def get_contract_open_order(
        self,
        product_symbol: str = None,
        type: str = None,
        order_state: str = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param type: str (limit, market, trailing)
        :param order_state: str (all(default), partially_filled)
        :param limit: int
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMART, product_symbol)
        if type is not None:
            payload["type"] = type
        if order_state is not None:
            payload["order_state"] = order_state
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=FuturesTrade.GET_ALL_OPEN_ORDERS,
            query=payload,
        )

        return res

    def get_contract_position(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMART, product_symbol)

        res = self._request(
            method="GET",
            path=FuturesTrade.GET_CURRENT_POSITION,
            query=payload,
        )
        return res

    def get_contract_trade(
        self,
        product_symbol: str,
        start_time: str = None,
        end_time: str = None,
    ):
        """
        :param product_symbol: str
        :param start_time: str
        :param end_time: str
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BITMART, product_symbol),
        }
        if start_time is not None:
            payload["start_time"] = start_time
        if end_time is not None:
            payload["end_time"] = end_time

        res = self._request(
            method="GET",
            path=FuturesTrade.GET_ORDER_TRADE,
            query=payload,
        )
        return res

    def get_contract_transaction_history(
        self,
        product_symbol: str = None,
        flow_type: int = None,
        start_time: int = None,
        end_time: int = None,
        page_size: int = None,
    ):
        """
        :param product_symbol: str
        :param flow_type: int (0 = All (default), 1 = Transfer, 2 = Realized PNL, 3 = Funding Fee, 4 = Commission Fee, 5 = Liquidation)
        :param start_time: int
        :param end_time: int
        :param page_size: int
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BITMART, product_symbol)
        if flow_type is not None:
            payload["flow_type"] = flow_type
        if start_time is not None:
            payload["start_time"] = start_time
        if end_time is not None:
            payload["end_time"] = end_time
        if page_size is not None:
            payload["page_size"] = page_size

        res = self._request(
            method="GET",
            path=FuturesTrade.GET_TRANSACTION_HISTORY,
            query=payload,
        )
        return res

    def get_contract_transfer_list(
        self,
        page: int,
        limit: int,
        currency: str = None,
        start_time: int = None,
        end_time: int = None,
    ):
        """
        :param page: int
        :param limit: int
        :param currency: str
        :param start_time: int
        :param end_time: int
        """
        payload = {
            "page": page,
            "limit": limit,
        }
        if currency is not None:
            payload["currency"] = currency
        if start_time is not None:
            payload["time_start"] = start_time
        if end_time is not None:
            payload["time_end"] = end_time

        res = self._request(
            method="POST",
            path=FuturesTrade.GET_TRANSFER_LIST,
            query=payload,
        )
        return res



from ._http_manager import HTTPManager
from .endpoints.trade import SpotTrade
from ...utils.common import Common


class TradeHTTP(HTTPManager):
    async def place_order(
        self,
        product_symbol: str,
        side: str,
        type_: str,
        size: str = None,
        funds: str = None,
        price: str = None,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        timeInForce: str = None,
        cancelAfter: int = None,
        postOnly: bool = None,
        hidden: bool = None,
        iceberg: bool = None,
        visibleSize: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol (e.g., "BTC-USDT-SPOT")
        :param side: str - "buy" or "sell"
        :param type_: str - "limit" or "market"
        :param size: str - Order size (required for limit orders, optional for market orders)
        :param funds: str - Order funds (for market orders, use either size or funds)
        :param price: str - Order price (required for limit orders)
        :param clientOid: str - Client order ID (recommended to use UUID)
        :param stp: str - Self trade prevention: "DC", "CO", "CN", "CB"
        :param tags: str - Order tag (max 20 ASCII characters)
        :param remark: str - Order remark (max 20 ASCII characters)
        :param timeInForce: str - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
        :param cancelAfter: int - Cancel after n seconds (for GTT strategy)
        :param postOnly: bool - Passive order label (disabled for IOC/FOK)
        :param hidden: bool - Hidden order (not shown in order book)
        :param iceberg: bool - Iceberg order
        :param visibleSize: str - Maximum visible quantity in iceberg orders
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp (equal to KC-API-TIMESTAMP)
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol),
            "side": side,
            "type": type_,
        }

        if size is not None:
            payload["size"] = size
        if funds is not None:
            payload["funds"] = funds
        if price is not None:
            payload["price"] = price
        if clientOid is not None:
            payload["clientOid"] = clientOid
        if stp is not None:
            payload["stp"] = stp
        if tags is not None:
            payload["tags"] = tags
        if remark is not None:
            payload["remark"] = remark
        if timeInForce is not None:
            payload["timeInForce"] = timeInForce
        if cancelAfter is not None:
            payload["cancelAfter"] = cancelAfter
        if postOnly is not None:
            payload["postOnly"] = postOnly
        if hidden is not None:
            payload["hidden"] = hidden
        if iceberg is not None:
            payload["iceberg"] = iceberg
        if visibleSize is not None:
            payload["visibleSize"] = visibleSize
        if allowMaxTimeWindow is not None:
            payload["allowMaxTimeWindow"] = allowMaxTimeWindow
        if clientTimestamp is not None:
            payload["clientTimestamp"] = clientTimestamp

        res = await self._request(
            method="POST",
            path=SpotTrade.PLACE_ORDER,
            query=payload,
        )
        return res

    async def place_market_order(
        self,
        product_symbol: str,
        side: str,
        size: str = None,
        funds: str = None,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param side: str - "buy" or "sell"
        :param size: str - Order size (use either size or funds)
        :param funds: str - Order funds (use either size or funds)
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_order(
            product_symbol=product_symbol,
            side=side,
            type_="market",
            size=size,
            funds=funds,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_market_buy_order(
        self,
        product_symbol: str,
        size: str = None,
        funds: str = None,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param size: str - Order size (use either size or funds)
        :param funds: str - Order funds (use either size or funds)
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_market_order(
            product_symbol=product_symbol,
            side="buy",
            size=size,
            funds=funds,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_market_sell_order(
        self,
        product_symbol: str,
        size: str = None,
        funds: str = None,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param size: str - Order size (use either size or funds)
        :param funds: str - Order funds (use either size or funds)
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_market_order(
            product_symbol=product_symbol,
            side="sell",
            size=size,
            funds=funds,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_limit_order(
        self,
        product_symbol: str,
        side: str,
        size: str,
        price: str,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        timeInForce: str = "GTC",
        cancelAfter: int = None,
        postOnly: bool = None,
        hidden: bool = None,
        iceberg: bool = None,
        visibleSize: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param side: str - "buy" or "sell"
        :param size: str - Order size
        :param price: str - Order price
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param timeInForce: str - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
        :param cancelAfter: int - Cancel after n seconds (for GTT strategy)
        :param postOnly: bool - Passive order label
        :param hidden: bool - Hidden order
        :param iceberg: bool - Iceberg order
        :param visibleSize: str - Maximum visible quantity in iceberg orders
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_order(
            product_symbol=product_symbol,
            side=side,
            type_="limit",
            size=size,
            price=price,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            timeInForce=timeInForce,
            cancelAfter=cancelAfter,
            postOnly=postOnly,
            hidden=hidden,
            iceberg=iceberg,
            visibleSize=visibleSize,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_limit_buy_order(
        self,
        product_symbol: str,
        size: str,
        price: str,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        timeInForce: str = "GTC",
        cancelAfter: int = None,
        postOnly: bool = None,
        hidden: bool = None,
        iceberg: bool = None,
        visibleSize: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param size: str - Order size
        :param price: str - Order price
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param timeInForce: str - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
        :param cancelAfter: int - Cancel after n seconds (for GTT strategy)
        :param postOnly: bool - Passive order label
        :param hidden: bool - Hidden order
        :param iceberg: bool - Iceberg order
        :param visibleSize: str - Maximum visible quantity in iceberg orders
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_limit_order(
            product_symbol=product_symbol,
            side="buy",
            size=size,
            price=price,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            timeInForce=timeInForce,
            cancelAfter=cancelAfter,
            postOnly=postOnly,
            hidden=hidden,
            iceberg=iceberg,
            visibleSize=visibleSize,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_limit_sell_order(
        self,
        product_symbol: str,
        size: str,
        price: str,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        timeInForce: str = "GTC",
        cancelAfter: int = None,
        postOnly: bool = None,
        hidden: bool = None,
        iceberg: bool = None,
        visibleSize: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param size: str - Order size
        :param price: str - Order price
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param timeInForce: str - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
        :param cancelAfter: int - Cancel after n seconds (for GTT strategy)
        :param postOnly: bool - Passive order label
        :param hidden: bool - Hidden order
        :param iceberg: bool - Iceberg order
        :param visibleSize: str - Maximum visible quantity in iceberg orders
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_limit_order(
            product_symbol=product_symbol,
            side="sell",
            size=size,
            price=price,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            timeInForce=timeInForce,
            cancelAfter=cancelAfter,
            postOnly=postOnly,
            hidden=hidden,
            iceberg=iceberg,
            visibleSize=visibleSize,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_post_only_limit_order(
        self,
        product_symbol: str,
        side: str,
        size: str,
        price: str,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        timeInForce: str = "GTC",
        cancelAfter: int = None,
        hidden: bool = None,
        iceberg: bool = None,
        visibleSize: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param side: str - "buy" or "sell"
        :param size: str - Order size
        :param price: str - Order price
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param timeInForce: str - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
        :param cancelAfter: int - Cancel after n seconds (for GTT strategy)
        :param hidden: bool - Hidden order
        :param iceberg: bool - Iceberg order
        :param visibleSize: str - Maximum visible quantity in iceberg orders
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_limit_order(
            product_symbol=product_symbol,
            side=side,
            size=size,
            price=price,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            timeInForce=timeInForce,
            cancelAfter=cancelAfter,
            postOnly=True,  # Set postOnly to True
            hidden=hidden,
            iceberg=iceberg,
            visibleSize=visibleSize,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_post_only_limit_buy_order(
        self,
        product_symbol: str,
        size: str,
        price: str,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        timeInForce: str = "GTC",
        cancelAfter: int = None,
        hidden: bool = None,
        iceberg: bool = None,
        visibleSize: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param size: str - Order size
        :param price: str - Order price
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param timeInForce: str - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
        :param cancelAfter: int - Cancel after n seconds (for GTT strategy)
        :param hidden: bool - Hidden order
        :param iceberg: bool - Iceberg order
        :param visibleSize: str - Maximum visible quantity in iceberg orders
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_post_only_limit_order(
            product_symbol=product_symbol,
            side="buy",
            size=size,
            price=price,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            timeInForce=timeInForce,
            cancelAfter=cancelAfter,
            hidden=hidden,
            iceberg=iceberg,
            visibleSize=visibleSize,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_post_only_limit_sell_order(
        self,
        product_symbol: str,
        size: str,
        price: str,
        clientOid: str = None,
        stp: str = None,
        tags: str = None,
        remark: str = None,
        timeInForce: str = "GTC",
        cancelAfter: int = None,
        hidden: bool = None,
        iceberg: bool = None,
        visibleSize: str = None,
        allowMaxTimeWindow: int = None,
        clientTimestamp: int = None,
    ):
        """
        :param product_symbol: str - Product symbol
        :param size: str - Order size
        :param price: str - Order price
        :param clientOid: str - Client order ID
        :param stp: str - Self trade prevention
        :param tags: str - Order tag
        :param remark: str - Order remark
        :param timeInForce: str - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
        :param cancelAfter: int - Cancel after n seconds (for GTT strategy)
        :param hidden: bool - Hidden order
        :param iceberg: bool - Iceberg order
        :param visibleSize: str - Maximum visible quantity in iceberg orders
        :param allowMaxTimeWindow: int - Order timeout in milliseconds
        :param clientTimestamp: int - Client timestamp
        """
        return await self.place_post_only_limit_order(
            product_symbol=product_symbol,
            side="sell",
            size=size,
            price=price,
            clientOid=clientOid,
            stp=stp,
            tags=tags,
            remark=remark,
            timeInForce=timeInForce,
            cancelAfter=cancelAfter,
            hidden=hidden,
            iceberg=iceberg,
            visibleSize=visibleSize,
            allowMaxTimeWindow=allowMaxTimeWindow,
            clientTimestamp=clientTimestamp,
        )

    async def place_batch_orders(
        self,
        orders: list,
    ):
        """
        :param orders: list - List of order dictionaries, each containing:
            - symbol: str (required) - Trading pair symbol
            - type: str (required) - "limit" or "market"
            - side: str (required) - "buy" or "sell"
            - size: str (required for limit, optional for market) - Order size
            - funds: str (optional for market) - Order funds (use either size or funds for market)
            - price: str (required for limit) - Order price
            - clientOid: str (optional) - Client order ID
            - stp: str (optional) - Self trade prevention: "CN", "CO", "CB", "DC"
            - tags: str (optional) - Order tag (max 20 ASCII characters)
            - remark: str (optional) - Order remark (max 20 ASCII characters)
            - timeInForce: str (optional) - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
            - cancelAfter: int (optional) - Cancel after n seconds (for GTT strategy)
            - postOnly: bool (optional) - Passive order label
            - hidden: bool (optional) - Hidden order
            - iceberg: bool (optional) - Iceberg order
            - visibleSize: str (optional) - Maximum visible quantity in iceberg orders

        Example:
            orders = [
                {
                    "symbol": "BTC-USDT",
                    "type": "limit",
                    "side": "buy",
                    "size": "0.001",
                    "price": "45000",
                    "clientOid": "uuid1"
                },
                {
                    "symbol": "ETH-USDT",
                    "type": "market",
                    "side": "sell",
                    "size": "0.01",
                    "clientOid": "uuid2"
                }
            ]
        """
        if not orders:
            raise ValueError("Orders list cannot be empty")

        if len(orders) > 20:
            raise ValueError("Maximum 20 orders can be placed simultaneously")

        processed_orders = []
        for order in orders:
            processed_order = order.copy()
            if "symbol" in processed_order:
                processed_order["symbol"] = self.ptm.get_exchange_symbol(Common.KUCOIN, processed_order["symbol"])
            processed_orders.append(processed_order)

        payload = {"orderList": processed_orders}

        res = await self._request(
            method="POST",
            path=SpotTrade.BATCH_ORDERS,
            query=payload,
        )
        return res

    async def place_batch_limit_orders(
        self,
        orders: list,
    ):
        """
        :param orders: list - List of limit order dictionaries, each containing:
            - symbol: str (required) - Trading pair symbol
            - side: str (required) - "buy" or "sell"
            - size: str (required) - Order size
            - price: str (required) - Order price
            - clientOid: str (optional) - Client order ID
            - stp: str (optional) - Self trade prevention
            - tags: str (optional) - Order tag
            - remark: str (optional) - Order remark
            - timeInForce: str (optional) - "GTC", "GTT", "IOC", "FOK" (default: "GTC")
            - cancelAfter: int (optional) - Cancel after n seconds (for GTT strategy)
            - postOnly: bool (optional) - Passive order label
            - hidden: bool (optional) - Hidden order
            - iceberg: bool (optional) - Iceberg order
            - visibleSize: str (optional) - Maximum visible quantity in iceberg orders
        """
        # Add type="limit" to all orders
        processed_orders = []
        for order in orders:
            processed_order = order.copy()
            processed_order["type"] = "limit"
            processed_orders.append(processed_order)

        return await self.place_batch_orders(processed_orders)

    async def place_batch_market_orders(
        self,
        orders: list,
    ):
        """
        :param orders: list - List of market order dictionaries, each containing:
            - symbol: str (required) - Trading pair symbol
            :param side: str (required) - "buy" or "sell"
            - size: str (optional) - Order size (use either size or funds)
            - funds: str (optional) - Order funds (use either size or funds)
            - clientOid: str (optional) - Client order ID
            - stp: str (optional) - Self trade prevention
            - tags: str (optional) - Order tag
            - remark: str (optional) - Order remark
        """
        # Add type="market" to all orders
        processed_orders = []
        for order in orders:
            processed_order = order.copy()
            processed_order["type"] = "market"
            processed_orders.append(processed_order)

        return await self.place_batch_orders(processed_orders)

    async def cancel_order(
        self,
        orderId: str,
        product_symbol: str,
    ):
        """
        :param orderId: str - The unique order ID generated by the trading system
        :param product_symbol: str - Product symbol (e.g., "BTC-USDT-SPOT")
        """
        # Format the path with orderId
        path = SpotTrade.CANCEL_ORDER.format(orderId=orderId)

        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol),
        }

        res = await self._request(
            method="DELETE",
            path=path,
            query=payload,
        )
        return res

    async def cancel_all_orders_by_symbol(
        self,
        product_symbol: str,
    ):
        """
        :param product_symbol: str - Product symbol
        """
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol),
        }
        res = await self._request(
            method="DELETE",
            path=SpotTrade.CANCEL_ALL_ORDERS_BY_SYMBOL,
            query=payload,
        )
        return res

    async def cancel_all_orders(
        self,
    ):
        res = await self._request(
            method="DELETE",
            path=SpotTrade.CANCEL_ALL_ORDERS,
        )
        return res

    async def get_open_orders(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str - Optional product symbol filter
        """
        payload = {}
        if product_symbol:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol)

        res = await self._request(
            method="GET",
            path=SpotTrade.GET_OPEN_ORDERS,
            query=payload,
        )
        return res

    async def get_trade_history(
        self,
        product_symbol: str = None,
        orderId: str = None,
        startAt: int = None,
        endAt: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str - Optional product symbol filter
        :param orderId: str - Optional order ID filter
        :param startAt: int - Start time (milliseconds)
        :param endAt: int - End time (milliseconds)
        :param limit: int - Number of records to return
        """
        payload = {}
        if product_symbol:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.KUCOIN, product_symbol)
        if orderId:
            payload["orderId"] = orderId
        if startAt:
            payload["startAt"] = startAt
        if endAt:
            payload["endAt"] = endAt
        if limit:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SpotTrade.GET_TRADE_HISTORY,
            query=payload,
        )
        return res



from ._http_manager import HTTPManager
from .endpoints.account import Account
from ..utils.common import Common


class AccountHTTP(HTTPManager):
    def get_account_instruments(
        self,
        instType: str,
        product_symbol: str = None,
        instFamily: str = None,
        uly: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param product_symbol: str Only applicable to FUTURES/SWAP/OPTION.If instType is OPTION, either uly or instFamily is required.
        :param instFamily: str Only applicable to FUTURES/SWAP/OPTION. If instType is OPTION, either uly or instFamily is required.
        :param uly: str
        """
        payload = {
            "instType": instType,
        }
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if instFamily is not None:
            payload["instFamily"] = instFamily
        if uly is not None:
            payload["uly"] = uly

        res = self._request(
            method="GET",
            path=Account.GET_INSTRUMENTS,
            query=payload,
        )
        return res

    def get_account_balance(
        self,
        ccy: str = None,
    ):
        """
        :param ccy: str
        """
        payload = {}
        if ccy is not None:
            coinName = ",".join(ccy)
            payload = {
                "ccy": coinName,
            }

        res = self._request(
            method="GET",
            path=Account.ACCOUNT_INFO,
            query=payload,
        )
        return res

    def get_positions(
        self,
        instType: str = None,
        product_symbol: str = None,
    ):
        """
        :param instType: str (MARGIN, SWAP, FUTURES, OPTION) instId will be checked against instType when both parameters are passed.
        :param product_symbol: str
        """
        payload = {}
        if instType is not None:
            payload["instType"] = instType
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)

        res = self._request(
            method="GET",
            path=Account.POSITION_INFO,
            query=payload,
        )
        return res

    def get_positions_history(
        self,
        instType: str = None,
        product_symbol: str = None,
        mgnMode: str = None,
        type: str = None,
        after: str = None,
        before: str = None,
        limit: str = None,
    ):
        """
        :param instType: str (MARGIN, SWAP, FUTURES, OPTION)
        :param product_symbol: str
        :param mgnMode: str (cross, isolated)
        :param type: str (1: Close position partially; 2: Close all; 3: Liquidation; 4: Partial liquidation; 5: ADL;)
        :param after: str
        :param before: str
        :param limit: str
        """
        payload = {}
        if instType is not None:
            payload["instType"] = instType
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if mgnMode is not None:
            payload["mgnMode"] = mgnMode
        if type is not None:
            payload["type"] = type
        if after is not None:
            payload["after"] = after
        if before is not None:
            payload["before"] = before
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Account.POSITIONS_HISTORY,
            query=payload,
        )
        return res

    def get_position_risk(
        self,
        instType: str = None,
    ):
        """
        :param instType: str (MARGIN, SWAP, FUTURES, OPTION)
        """
        payload = {}
        if instType is not None:
            payload["instType"] = instType

        res = self._request(
            method="GET",
            path=Account.POSITION_RISK,
            query=payload,
        )
        return res

    def get_account_bills(
        self,
        instType: str = None,
        product_symbol: str = None,
        ccy: str = None,
        mgnMode: str = None,
        ctType: str = None,
        type: str = None,
        subType: str = None,
        begin: str = None,
        end: str = None,
        limit: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param product_symbol: str
        :param ccy: str
        :param mgnMode: str (cross, isolated)
        :param ctType: str
        :param type: str
        :param subType: str
        :param begin: str
        :param end: str
        :param limit: str
        """
        payload = {}
        if instType is not None:
            payload["instType"] = instType
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ccy is not None:
            payload["ccy"] = ccy
        if mgnMode is not None:
            payload["mgnMode"] = mgnMode
        if ctType is not None:
            payload["ctType"] = ctType
        if type is not None:
            payload["type"] = type
        if subType is not None:
            payload["subType"] = subType
        if begin is not None:
            payload["begin"] = begin
        if end is not None:
            payload["end"] = end
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Account.BILLS_DETAIL,
            query=payload,
        )
        return res

    def get_account_bills_archive(
        self,
        instType: str = None,
        product_symbol: str = None,
        ccy: str = None,
        mgnMode: str = None,
        ctType: str = None,
        type: str = None,
        subType: str = None,
        begin: str = None,
        end: str = None,
        limit: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param product_symbol: str
        :param ccy: str
        :param mgnMode: str (cross, isolated)
        :param ctType: str
        :param type: str
        :param subType: str
        :param begin: str
        :param end: str
        :param limit: str
        """
        payload = {}
        if instType is not None:
            payload["instType"] = instType
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ccy is not None:
            payload["ccy"] = ccy
        if mgnMode is not None:
            payload["mgnMode"] = mgnMode
        if ctType is not None:
            payload["ctType"] = ctType
        if type is not None:
            payload["type"] = type
        if subType is not None:
            payload["subType"] = subType
        if begin is not None:
            payload["begin"] = begin
        if end is not None:
            payload["end"] = end
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Account.BILLS_ARCHIVE,
            query=payload,
        )
        return res

    def get_account_bills_history_archive(
        self,
        year: str,
        quarter: str,
    ):
        """
        :param year: str
        :param quarter: str
        """
        payload = {
            "year": year,
            "quarter": quarter,
        }

        res = self._request(
            method="GET",
            path=Account.BILLS_HISTORY_ARCHIVE,
            query=payload,
        )
        return res

    def post_account_bills_history_archive(
        self,
        year: str,
        quarter: str,
    ):
        """
        :param year: str
        :param quarter: str
        """
        payload = {
            "year": year,
            "quarter": quarter,
        }

        res = self._request(
            method="POST",
            path=Account.BILLS_HISTORY_ARCHIVE,
            query=payload,
        )
        return res

    def get_account_config(self):
        res = self._request(
            method="GET",
            path=Account.ACCOUNT_CONFIG,
            query=None,
        )
        return res

    def set_position_mode(self, posMode: str):
        """
        :param posMode: str (long_short_mode)
        """
        payload = {
            "posMode": posMode,
        }

        res = self._request(
            method="POST",
            path=Account.POSITION_MODE,
            query=payload,
        )
        return res

    def set_leverage(
        self,
        lever: str,
        mgnMode: str,
        product_symbol: str = None,
        ccy: str = None,
        posSide: str = None,
    ):
        """
        :param lever: str
        :param mgnMode: str (cross, isolated), Can only be cross if ccy is passed.
        :param product_symbol: str Under cross mode, either instId or ccy is required; if both are passed, instId will be used by default.
        :param ccy: str Only applicable to cross MARGIN of Spot mode/Multi-currency margin/Portfolio margin
        :param posSide: str Only required when margin mode is isolated in long/short mode for FUTURES/SWAP.
        """
        payload = {
            "lever": lever,
            "mgnMode": mgnMode,
        }
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ccy is not None:
            payload["ccy"] = ccy
        if posSide is not None:
            payload["posSide"] = posSide

        res = self._request(
            method="POST",
            path=Account.SET_LEVERAGE,
            query=payload,
        )
        return res

    def get_max_order_size(
        self,
        product_symbol: str,
        tdMode: str,
        ccy: str = None,
        px: str = None,
        leverage: str = None,
    ):
        """
        :param product_symbol: str
        :param tdMode: str (cross, isolated, cash, spot_isolated)
        :param ccy: str Currency used for margin Applicable to isolated MARGIN and cross MARGIN orders in Spot and futures mode.
        :param px: str
        :param leverage: str
        """
        payload = {
            "instId": self.ptm.get_exchange_symbol(Common.OKX, product_symbol),
            "tdMode": tdMode,
        }
        if ccy is not None:
            payload["ccy"] = ccy
        if px is not None:
            payload["px"] = px
        if leverage is not None:
            payload["leverage"] = leverage

        res = self._request(
            method="GET",
            path=Account.MAX_TRADE_SIZE,
            query=payload,
        )
        return res

    def get_max_avail_size(
        self,
        product_symbol: str,
        tdMode: str,
        ccy: str = None,
        reduceOnly: str = None,
        px: str = None,
    ):
        """
        :param product_symbol: str
        :param tdMode: str (cross, isolated, cash, spot_isolated)
        :param ccy: str Applicable to isolated MARGIN and cross MARGIN in Spot and futures mode.
        :param reduceOnly: str Whether to reduce position only Only applicable to MARGIN
        :param px: str Only applicable to reduceOnly MARGIN.
        """
        payload = {
            "instId": self.ptm.get_exchange_symbol(Common.OKX, product_symbol),
            "tdMode": tdMode,
        }
        if ccy is not None:
            payload["ccy"] = ccy
        if reduceOnly is not None:
            payload["reduceOnly"] = reduceOnly
        if px is not None:
            payload["px"] = px

        res = self._request(
            method="GET",
            path=Account.MAX_AVAIL_SIZE,
            query=payload,
        )
        return res

    def get_leverage(
        self,
        mgnMode: str,
        product_symbol: str = None,
        ccy: str = None,
    ):
        """
        :param mgnMode: str (cross, isolated)
        :param product_symbol: str
        :param ccy: str used for getting leverage of currency level. Applicable to cross MARGIN of Spot mode/Multi-currency margin/Portfolio margin. Supported single currency or multiple currencies (no more than 20) separated with comma.
        """
        payload = {
            "mgnMode": mgnMode,
        }
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ccy is not None:
            payload["ccy"] = ccy

        res = self._request(
            method="GET",
            path=Account.GET_LEVERAGE,
            query=payload,
        )
        return res

    def get_adjust_leverage(
        self,
        instType: str,
        mgnMode: str,
        lever: str,
        product_symbol: str = None,
        ccy: str = None,
        posSide: str = None,
    ):
        """
        :param instType: str (MARGIN, SWAP, FUTURES)
        :param mgnMode: str (cross, isolated)
        :param lever: str
        :param product_symbol: str
        :param ccy: str
        :param posSide: str (long, short)
        """
        payload = {
            "instType": instType,
            "mgnMode": mgnMode,
            "lever": lever,
        }
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ccy is not None:
            payload["ccy"] = ccy
        if posSide is not None:
            payload["posSide"] = posSide

        res = self._request(
            method="GET",
            path=Account.GET_ADJUST_LEVERAGE,
            query=payload,
        )
        return res

    def get_max_loan(
        self,
        mgnMode: str,
        product_symbol: str = None,
        ccy: str = None,
        mgnCcy: str = None,
    ):
        """
        :param mgnMode: str (cross, isolated)
        :param product_symbol: str
        :param ccy: str
        :param mgnCcy: str
        """
        payload = {
            "mgnMode": mgnMode,
        }
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if ccy is not None:
            payload["ccy"] = ccy
        if mgnCcy is not None:
            payload["mgnCcy"] = mgnCcy

        res = self._request(
            method="GET",
            path=Account.MAX_LOAN,
            query=payload,
        )
        return res

    def get_fee_rates(
        self,
        instType: str,
        ruleType: str = None,
        product_symbol: str = None,
        uly: str = None,
        instFamily: str = None,
    ):
        """
        :param instType: str (SPOT, MARGIN, SWAP, FUTURES, OPTION)
        :param ruleType: str Trading rule types normal: normal trading pre_market: pre-market trading ruleType can not be passed through together with product_symbol/instFamily/uly
        :param product_symbol: str
        :param uly: str
        :param instFamily: str
        """
        payload = {
            "instType": instType,
        }
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if uly is not None:
            payload["uly"] = uly
        if instFamily is not None:
            payload["instFamily"] = instFamily
        if ruleType is not None:
            payload["ruleType"] = ruleType

        res = self._request(
            method="GET",
            path=Account.FEE_RATES,
            query=payload,
        )
        return res

    def get_interest_accrued(
        self,
        ccy: str = None,
        product_symbol: str = None,
        mgnMode: str = None,
        after: str = None,
        before: str = None,
        limit: str = None,
    ):
        """
        :param ccy: str
        :param product_symbol: str
        :param mgnMode: str (cross, isolated)
        :param after: str
        :param before: str
        :param limit: str
        """
        payload = {}
        if ccy is not None:
            payload["ccy"] = ccy
        if product_symbol is not None:
            payload["instId"] = self.ptm.get_exchange_symbol(Common.OKX, product_symbol)
        if mgnMode is not None:
            payload["mgnMode"] = mgnMode
        if after is not None:
            payload["after"] = after
        if before is not None:
            payload["before"] = before
        if limit is not None:
            payload["limit"] = limit

        res = self._request(
            method="GET",
            path=Account.INTEREST_ACCRUED,
            query=payload,
        )
        return res

    def get_interest_rate(
        self,
        ccy: str = None,
    ):
        """
        :param ccy: str
        """
        payload = {}
        if ccy is not None:
            payload["ccy"] = ccy

        res = self._request(
            method="GET",
            path=Account.INTEREST_RATE,
            query=payload,
        )
        return res

    def set_greeks(
        self,
        greeksType: str,
    ):
        """
        :param greeksType: str PA: Greeks in coins, BS: Black-Scholes Greeks in dollars
        """
        payload = {
            "greeksType": greeksType,
        }

        res = self._request(
            method="POST",
            path=Account.SET_GREEKS,
            query=payload,
        )
        return res

    def set_isolated_mode(
        self,
        type: str,
    ):
        """
        :param type: str (MARGIN, CONTRACTS)
        """
        payload = {
            "isoMode": "automatic",
            "type": type,
        }

        res = self._request(
            method="POST",
            path=Account.SET_ISOLATED_MODE,
            query=payload,
        )
        return res

    def get_max_withdrawal(
        self,
        ccy: str = None,
    ):
        """
        :param ccy: str
        """
        payload = {}
        if ccy is not None:
            ccyName = ",".join(ccy)
            payload = {
                "ccy": ccyName,
            }

        res = self._request(
            method="GET",
            path=Account.MAX_WITHDRAWAL,
            query=payload,
        )
        return res

    def get_interest_limits(
        self,
        ccy: str = None,
    ):
        """
        :param ccy: str
        """
        payload = {}
        if ccy is not None:
            payload["ccy"] = ccy

        res = self._request(
            method="GET",
            path=Account.INTEREST_LIMITS,
            query=payload,
        )
        return res

    def set_auto_loan(
        self,
        autoLoan: bool,
    ):
        """
        :param autoLoan: bool
        """
        kwargs = {
            "autoLoan": autoLoan,
        }

        res = self._request(
            method="POST",
            path=Account.SET_AUTO_LOAN,
            query=kwargs,
        )
        return res



from ._http_manager import HTTPManager
from .endpoints.asset import Asset


class AssetHTTP(HTTPManager):
    async def get_currencies(
        self,
        ccy: str = None,
    ):
        """
        :param ccy: str
        """
        payload = {}
        if ccy is not None:
            ccyName = ",".join(ccy)
            payload = {
                "ccy": ccyName,
            }

        res = await self._request(
            method="GET",
            path=Asset.CURRENCY_INFO,
            query=payload,
        )
        return res

    async def get_balances(
        self,
        ccy: str = None,
    ):
        """
        :param ccy: str
        """
        payload = {}
        if ccy is not None:
            ccyName = ",".join(ccy)
            payload = {
                "ccy": ccyName,
            }

        res = await self._request(
            method="GET",
            path=Asset.GET_BALANCES,
            query=payload,
        )
        return res

    async def get_asset_valuation(
        self,
        ccy: str = None,
    ):
        """
        :param ccy: str
        """
        payload = {}
        if ccy is not None:
            ccyName = ",".join(ccy)
            payload = {
                "ccy": ccyName,
            }

        res = await self._request(
            method="GET",
            path=Asset.ASSET_VALUATION,
            query=payload,
        )
        return res

    async def funds_transfer(
        self,
        ccy: str,
        amt: str,
        from_account: str,
        to_account: str,
        type: str = None,
        subAcct: str = None,
        loanTrans: str = None,
    ):
        """
        :param ccy: str
        :param amt: str
        :param from_account: str
        :param to_account: str
        :param type: str
        :param subAcct: str
        :param loanTrans: str
        """
        account_map = {
            "FUND": "6",
            "TRADING": "18",
        }

        payload = {
            "ccy": ccy,
            "amt": amt,
            "from": account_map.get(from_account),
            "to": account_map.get(to_account),
        }

        if type is not None:
            payload["type"] = type
        if subAcct is not None:
            payload["subAcct"] = subAcct
        if loanTrans is not None:
            payload["loanTrans"] = loanTrans

        res = await self._request(
            method="POST",
            path=Asset.FUNDS_TRANSFER,
            query=payload,
        )
        return res

    async def get_transfer_state(
        self,
        transId: str = None,
        clientId: str = None,
        type: str = None,
    ):
        """
        :param transId: str
        :param clientId: str
        :param type: str
        """
        payload = {}
        if transId is not None:
            payload["transId"] = transId
        if clientId is not None:
            payload["clientId"] = clientId
        if type is not None:
            payload["type"] = type

        res = await self._request(
            method="GET",
            path=Asset.TRANSFER_STATE,
            query=payload,
        )
        return res

    async def get_bills(
        self,
        type: str = None,
        clientId: str = None,
        after: str = None,
        before: str = None,
        limit: str = None,
    ):
        """
        :param type: str
        :param clientId: str
        :param after: str
        :param before: str
        :param limit: str
        """
        payload = {}
        if type is not None:
            payload["type"] = type
        if clientId is not None:
            payload["clientId"] = clientId
        if after is not None:
            payload["after"] = after
        if before is not None:
            payload["before"] = before
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=Asset.BILLS_INFO,
            query=payload,
        )
        return res

    async def get_deposit_address(
        self,
        ccy: str,
    ):
        """
        :param ccy: str
        """
        payload = {
            "ccy": ccy,
        }

        res = await self._request(
            method="GET",
            path=Asset.DEPOSIT_ADDRESS,
            query=payload,
        )
        return res

    async def get_deposit_history(
        self,
        ccy: str = None,
        depId: str = None,
        fromWdId: str = None,
        txId: str = None,
        type: str = None,
        state: str = None,
        after: str = None,
        before: str = None,
        limit: str = None,
    ):
        """
        :param ccy: str
        :param depId: str
        :param fromWdId: str
        :param txId: str
        :param type: str
        :param state: str
        :param after: str
        :param before: str
        :param limit: str
        """
        payload = {}
        if ccy is not None:
            payload["ccy"] = ccy
        if depId is not None:
            payload["depId"] = depId
        if fromWdId is not None:
            payload["fromWdId"] = fromWdId
        if txId is not None:
            payload["txId"] = txId
        if type is not None:
            payload["type"] = type
        if state is not None:
            payload["state"] = state
        if after is not None:
            payload["after"] = after
        if before is not None:
            payload["before"] = before
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=Asset.DEPOSIT_HISTORY,
            query=payload,
        )
        return res

    async def withdrawal(
        self,
        ccy: str,
        amt: str,
        dest: str,
        toAddr: str,
        chain: str = None,
        areaCode: str = None,
        rcvrInfo: str = None,
    ):
        """
        :param ccy: str
        :param amt: str
        :param dest: str
        :param toAddr: str
        :param chain: str
        :param areaCode: str
        :param rcvrInfo: str
        """
        payload = {
            "ccy": ccy,
            "amt": amt,
            "dest": dest,
            "toAddr": toAddr,
        }
        if chain is not None:
            payload["chain"] = chain
        if areaCode is not None:
            payload["areaCode"] = areaCode
        if rcvrInfo is not None:
            payload["rcvrInfo"] = rcvrInfo

        res = await self._request(
            method="POST",
            path=Asset.WITHDRAWAL_COIN,
            query=payload,
        )
        return res

    async def cancel_withdrawal(
        self,
        wdId: str,
    ):
        """
        :param wdId: str
        """
        payload = {
            "wdId": wdId,
        }

        res = await self._request(
            method="POST",
            path=Asset.CANCEL_WITHDRAWAL,
            query=payload,
        )
        return res

    async def get_withdrawal_history(
        self,
        ccy: str = None,
        wdId: str = None,
        clientId: str = None,
        txId: str = None,
        type: str = None,
        state: str = None,
        after: str = None,
        before: str = None,
        limit: str = None,
    ):
        """
        :param ccy: str
        :param wdId: str
        :param clientId: str
        :param txId: str
        :param type: str
        :param state: str
        :param after: str
        :param before: str
        :param limit: str
        """
        payload = {}
        if ccy is not None:
            payload["ccy"] = ccy
        if wdId is not None:
            payload["wdId"] = wdId
        if clientId is not None:
            payload["clientId"] = clientId
        if txId is not None:
            payload["txId"] = txId
        if type is not None:
            payload["type"] = type
        if state is not None:
            payload["state"] = state
        if after is not None:
            payload["after"] = after
        if before is not None:
            payload["before"] = before
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=Asset.WITHDRAWAL_HISTORY,
            query=payload,
        )
        return res

    async def get_deposit_withdraw_status(
        self,
        wdId: str = None,
        txId: str = None,
        ccy: str = None,
        to: str = None,
        chain: str = None,
    ):
        """
        :param wdId: str
        :param txId: str
        :param ccy: str
        :param to: str
        :param chain: str
        """
        payload = {}
        if wdId is not None:
            payload["wdId"] = wdId
        if txId is not None:
            payload["txId"] = txId
        if ccy is not None:
            payload["ccy"] = ccy
        if to is not None:
            payload["to"] = to
        if chain is not None:
            payload["chain"] = chain

        res = await self._request(
            method="GET",
            path=Asset.GET_DEPOSIT_WITHDRAW_STATUS,
            query=payload,
        )
        return res

    async def get_exchange_list(self):
        res = await self._request(
            method="GET",
            path=Asset.EXCHANGE_LIST,
            query=None,
        )
        return res

    async def post_monthly_statement(
        self,
        month: str = None,
    ):
        """
        :param month: str (Jan)
        """
        payload = {}
        if month is not None:
            payload["month"] = month

        res = await self._request(
            method="POST",
            path=Asset.MONTHLY_STATEMENT,
            query=payload,
        )
        return res

    async def get_monthly_statement(
        self,
        month: str,
    ):
        """
        :param month: str (Jan)
        """
        payload = {
            "month": month,
        }

        res = await self._request(
            method="GET",
            path=Asset.MONTHLY_STATEMENT,
            query=payload,
        )
        return res

    async def get_convert_currencies(self):
        res = await self._request(
            method="GET",
            path=Asset.GET_CURRENCIES,
            query=None,
        )
        return res



from enum import Enum


class SpotTrade(str, Enum):
    #  https://api-cloud.bitmart.com
    SUBMIT_ORDER = "/spot/v2/submit_order"
    CANCEL_ORDER = "/spot/v3/cancel_order"
    CANCEL_ALL_ORDERS = "/spot/v4/cancel_all"
    NEW_MARGIN_ORDER = "/spot/v1/margin/submit_order"
    QUERY_ORDER_BY_ID = "/spot/v4/query/order"
    QUERY_ORDER_BY_CLIENT_ORDER_ID = "/spot/v4/query/client-order"
    CURRENT_OPEN_ORDERS = "/spot/v4/query/open-orders"
    ACCOUNT_ORDERS = "/spot/v4/query/history-orders"
    ACCOUNT_TRADE_LIST = "/spot/v4/query/trades"
    ORDER_TRADE_LIST = "/spot/v4/query/order-trades"

    def __str__(self) -> str:
        return self.value


class FuturesTrade(str, Enum):
    # https://api-cloud-v2.bitmart.com
    SUBMIT_ORDER = "/contract/private/submit-order"
    MODIFY_LIMIT_ORDER = "/contract/private/modify-limit-order"
    CANCEL_ORDER = "/contract/private/cancel-order"
    CANCEL_ALL_ORDERS = "/contract/private/cancel-orders"
    TRANSFER = "/account/v1/transfer-contract"
    SUBMIT_LEVERAGE = "/contract/private/submit-leverage"
    GET_ORDER_DETAIL = "/contract/private/order"
    GET_ORDER_HISTORY = "/contract/private/order-history"
    GET_ALL_OPEN_ORDERS = "/contract/private/get-open-orders"
    GET_CURRENT_POSITION = "/contract/private/position"
    GET_ORDER_TRADE = "/contract/private/trades"
    GET_TRANSACTION_HISTORY = "/contract/private/transaction-history"
    GET_TRANSFER_LIST = "/account/v1/transfer-contract-list"

    def __str__(self) -> str:
        return self.value



from ._http_manager import HTTPManager
from .endpoints.market import SpotMarket, FuturesMarket
from .enums import BinanceExchangeType
from ...utils.common import Common


class MarketHTTP(HTTPManager):
    async def get_spot_exchange_info(
        self,
        product_symbol: str = None,
        product_symbols: list = None,
        symbolStatus: str = None,
    ):
        """
        :param product_symbol: str
        :param product_symbols: list
        :param symbolStatus: str (Filters symbols that have this tradingStatus. Valid values: TRADING, HALT, BREAK Cannot be used in combination with symbols or symbol.)
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol)
        if product_symbols is not None:
            formatted_symbols = [self.ptm.get_exchange_symbol(Common.BINANCE, symbol) for symbol in product_symbols]
            payload["symbols"] = str(formatted_symbols).replace("'", '"')
        if symbolStatus is not None:
            payload["symbolStatus"] = symbolStatus

        res = await self._request(
            method="GET",
            path=SpotMarket.EXCHANGE_INFO,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_orderbook(
        self,
        product_symbol: str,
        limit: int = None,
    ):
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }
        if limit is not None:
            payload["limit"] = limit
        res = await self._request(
            method="GET",
            path=SpotMarket.ORDERBOOK,
            query=payload,
            signed=False,
        )
        return res

    async def get_spot_trades(
        self,
        product_symbol: str,
        limit: int = None,
    ):
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
        }
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SpotMarket.TRADES,
            query=payload,
            signed=False,
        )
        return res

    async def get_kline(
        self,
        product_symbol: str,
        interval: str,
        start_time: int = None,
        limit: int = None,
    ):
        payload = {
            "symbol": self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol),
            "interval": interval,
        }
        if start_time is not None:
            payload["startTime"] = start_time
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SpotMarket.KLINE
            if self.ptm.get_exchange_type(Common.BINANCE, product_symbol=product_symbol) == BinanceExchangeType.SPOT
            else FuturesMarket.KLINE,
            query=payload,
            signed=False,
        )
        return res

    async def get_futures_exchange_info(
        self,
    ):
        res = await self._request(
            method="GET",
            path=FuturesMarket.EXCHANGE_INFO,
            query=None,
            signed=False,
        )
        return res

    async def get_futures_ticker(
        self,
        product_symbol: str = None,
    ):
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol)

        res = await self._request(
            method="GET",
            path=FuturesMarket.BOOK_TICKER,
            query=payload,
            signed=False,
        )
        return res

    async def get_futures_premium_index(
        self,
        product_symbol: str = None,
    ):
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol)

        res = await self._request(
            method="GET",
            path=FuturesMarket.PREMIUM_INDEX,
            query=payload,
            signed=False,
        )
        return res

    async def get_futures_funding_rate(
        self,
        product_symbol: str = None,
        startTime: int = None,
        endTime: int = None,
        limit: int = None,
    ):
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINANCE, product_symbol)
        if startTime is not None:
            payload["startTime"] = startTime
        if endTime is not None:
            payload["endTime"] = endTime
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=FuturesMarket.FUNDING_RATE_HISTORY,
            query=payload,
            signed=False,
        )
        return res



from krex.utils.common import Common
from ._http_manager import HTTPManager
from .endpoints.account import SwapAccount


class AccountHTTP(HTTPManager):
    async def get_account_balance(self):
        payload = {}
        res = await self._request(
            method="GET",
            path=SwapAccount.ACCOUNT_BALANCE,
            query=payload,
        )
        return res

    async def get_open_positions(
        self,
        product_symbol: str = None,
    ):
        """
        :param product_symbol: str
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = self.ptm.get_exchange_symbol(Common.BINGX, product_symbol)

        res = await self._request(
            method="GET",
            path=SwapAccount.OPEN_POSITIONS,
            query=payload,
        )
        return res

    async def get_fund_flow(
        self,
        product_symbol: str = None,
        income_type: str = None,
        start_time: int = None,
        end_time: int = None,
        limit: int = None,
    ):
        """
        :param product_symbol: str
        :param income_type: str
        :param start_time: int
        :param end_time: int
        :param limit: int
        """
        payload = {}
        if product_symbol is not None:
            payload["symbol"] = product_symbol
        if income_type is not None:
            payload["incomeType"] = income_type
        if start_time is not None:
            payload["startTime"] = start_time
        if end_time is not None:
            payload["endTime"] = end_time
        if limit is not None:
            payload["limit"] = limit

        res = await self._request(
            method="GET",
            path=SwapAccount.FUND_FLOW,
            query=payload,
        )
        return res

    async def get_listen_key(self):
        if not self.session:
            await self.async_init()
        url = self.base_url + SwapAccount.LISTEN_KEY
        headers = {"X-BX-APIKEY": self.api_key}

        res = await self.session.post(url, headers=headers)
        data = res.json()
        return data.get("listenKey")

    async def keep_alive_listen_key(self, listen_key: str):
        """
        :param listen_key: str
        """
        payload = {
            "listenKey": listen_key,
        }

        res = await self._request(
            method="PUT",
            path=SwapAccount.LISTEN_KEY,
            query=payload,
        )
        return res



import requests
import json
import logging
import hmac
import base64
from dataclasses import dataclass, field
from ..product_table.manager import ProductTableManager
from ..utils.errors import FailedRequestError
from ..utils.helpers import generate_timestamp
from ..utils.common import Common


def _sign(message, secretKey):
    mac = hmac.new(
        bytes(secretKey, encoding="utf8"),
        bytes(message, encoding="utf-8"),
        digestmod="sha256",
    )
    d = mac.digest()
    return base64.b64encode(d).decode()


def pre_hash(timestamp, method, path, body):
    return str(timestamp) + str.upper(method) + path + body


def get_header(api_key, sign, timestamp, passphrase, flag):
    header = {
        "Content-Type": "application/json",
        "OK-ACCESS-KEY": api_key,
        "OK-ACCESS-SIGN": sign,
        "OK-ACCESS-TIMESTAMP": str(timestamp),
        "OK-ACCESS-PASSPHRASE": passphrase,
        "x-simulated-trading": flag,
    }
    return header


def parse_params_to_str(query):
    url = "?"
    for key, value in query.items():
        if value != "":
            url = url + str(key) + "=" + str(value) + "&"
    url = url[0:-1]
    return url


def get_header_no_sign(flag):
    header = {
        "Content-Type": "application/json",
        "x-simulated-trading": flag,
    }
    return header


@dataclass
class HTTPManager:
    api_key: str = field(default=None)
    api_secret: str = field(default=None)
    passphrase: str = field(default=None)
    flag: str = field(default="0")
    base_api: str = field(default="https://www.okx.com")
    max_retries: int = field(default=3)
    retry_delay: int = field(default=3)
    logger: logging.Logger = field(default=None)
    session: requests.Session = field(default_factory=requests.Session, init=False)
    ptm: ProductTableManager = field(init=False)
    preload_product_table: bool = field(default=True)

    def __post_init__(self):
        if self.logger is None:
            self._logger = logging.getLogger(__name__)
        else:
            self._logger = self.logger

        if self.preload_product_table:
            self.ptm = ProductTableManager.get_instance(Common.OKX)

    def _request(
        self,
        method: str,
        path: str,
        query: dict = None,
        signed: bool = True,
    ):
        if query is None:
            query = {}

        if method.upper() == "GET":
            path += parse_params_to_str(query)

        timestamp = generate_timestamp(iso_format=True)
        body = query if method.upper() == "POST" else ""
        body_str = json.dumps(body) if isinstance(body, (dict, list)) else body

        if signed:
            if not (self.api_key and self.api_secret and self.passphrase):
                raise ValueError("Signed request requires API Key and Secret and Passphrase.")
            sign = _sign(pre_hash(timestamp, method.upper(), path, body_str), self.api_secret)
            headers = get_header(self.api_key, sign, timestamp, self.passphrase, self.flag)
        else:
            headers = get_header_no_sign(self.flag)

        url = self.base_api + path

        try:
            if method.upper() == "GET":
                response = self.session.get(url, headers=headers)
            elif method.upper() == "POST":
                response = self.session.post(url, json=body, headers=headers)
            else:
                raise ValueError(f"Unsupported HTTP method: {method}")

            try:
                data = response.json()
            except Exception:
                data = {}

            if data.get("code", "0") != "0":
                code = data.get("code", "Unknown")
                error_message = data.get("msg", "Unknown error")
                raise FailedRequestError(
                    request=f"{method.upper()} {url} | Body: {query}",
                    message=f"OKX API Error: [{code}] {error_message}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            # If http status is not 2xx (like 403, 404)
            if not response.status_code // 100 == 2:
                raise FailedRequestError(
                    request=f"{method.upper()} {url} | Body: {query}",
                    message=f"HTTP Error {response.status_code}: {response.text}",
                    status_code=response.status_code,
                    time=timestamp,
                    resp_headers=response.headers,
                )

            return data
        except requests.exceptions.RequestException as e:
            raise FailedRequestError(
                request=f"{method.upper()} {url} | Body: {body}",
                message=f"Request failed: {str(e)}",
                status_code=response.status_code if response else "Unknown",
                time=timestamp,
                resp_headers=response.headers if response else None,
            )
